/*
  The help window will contain the following:
   - a menu bar for navigation
   - a toolbar for displaying buttons such as prev,next, search, hprev, etc.
   - a canvas to display the help text
*/

extern "C" void help_setting_i(const char *key, int value);
extern "C" void help_setting_s(const char *key, const char* value);
extern "C" void help(const char *subject);

namespace FX
{

typedef struct { const char *Vs; int Vi; } sipair_t;

class FXAPI FYHelpWin : public FXTopWindow
{
  FXDECLARE(FYHelpWin)
protected:
  FYHelpWin();
private:
  FYHelpWin(const FYHelpWin&);
  FYHelpWin &operator=(const FYHelpWin&);
public:
  FYHelpWin (FXApp* ap,const FXString& name, FXIcon *ic,FXIcon *mi,
            FXuint opts, FXint w,FXint h,
            FXint x=0,FXint y=0,
            FXint pl=0,FXint pr=0,FXint pt=0,FXint pb=0,
            FXint hs=0,FXint vs=0) ;
  virtual ~FYHelpWin();

// uu-chapsec
/* Chapters and sections will be accessible thru menus. */
private:
  FXMenuPane *chapters_pane;
  FXMenuPane *sections_pane;
  FXMenuTitle *sections_mtitle;

/* The chapter menu will never change. We don't need to keep track of
   that stuff. We simply provide a method to record them into the menu. */

public:  void menu_add_chapter(const char *title, int number);

/* The section menu will be set as a whole, rather than incremental 
   method calls. */
private: const sipair_t *sections_def;
public:  void set_section_menu(const sipair_t *);

public:
  FYGfxCanvas *canvas;
};
}

using namespace FX;

static void follow_link(int);
static void show_toc2(int);
static void show_toc0(int);


FXDEFMAP(FYHelpWin) FYHelpWinMap[]=
{
};


FXIMPLEMENT(FYHelpWin,FXTopWindow,FYHelpWinMap, ARRAYNUMBER(FYHelpWinMap))

FYHelpWin::FYHelpWin(){ flags|=FLAG_ENABLED|FLAG_SHOWN; }

/* Configuration variables and functions here */

static const char *CG_normal_font= "Sans";
static const char *CG_fixed_font= "Mono";
static int CG_normal_size = 12;
static int CG_normal_big  = 14;
static int CG_fixed_size  = 12;
static int CG_fixed_big  = 14;

static int CG_sechea1 = 14;
static int CG_sechea2 = 16;
static int CG_sechea3 = 18;
static int CG_sechea4 = 20;

static int CG_link = 0xff;
static int CG_red = 0xc67171;
static int CG_hilight= 0xb5cae1;
static int CG_code= 0xafe5b2;
static int CG_shell= 0xecabab;
static int CG_output= 0xade6ff;


void help_setting_i(const char *key, int value)
{
static const struct { const char *name; int *tgt; } opts[]=
{
  { "size",      &CG_normal_size },
  { "big",       &CG_normal_big },
  { "mono_size", &CG_fixed_size },
  { "mono_big",  &CG_fixed_big },
  { "header1",   &CG_sechea1 },
  { "header2",   &CG_sechea2 },
  { "header3",   &CG_sechea3 },
  { "header4",   &CG_sechea4 },
  { "link",      &CG_link },
  { "hilight",   &CG_hilight },
  { "red",       &CG_red },
  { "code",      &CG_code },
  { "shell",     &CG_shell },
  { "output",    &CG_output },
  { NULL, NULL }
};
  int i;
  for(i=0;opts[i].name;i++)
    if (!strcmp(opts[i].name,key)) { *(opts[i].tgt)= value; break; }
}

void help_setting_s(const char *key, const char* value)
{
static const struct { const char *name; const char **tgt; } opts[]=
{
  { "font",       &CG_normal_font },
  { "mono_font",  &CG_fixed_font },
  { NULL, NULL }
};
  int i;
  for(i=0;opts[i].name;i++)
    if (!strcmp(opts[i].name,key)) { *(opts[i].tgt)= strdup(value); break; }
}

FYHelpWin::FYHelpWin
           (FXApp* ap,const FXString& name, FXIcon *ic,FXIcon *mi,
            FXuint opts, FXint w,FXint h,
            FXint x,FXint y,
            FXint pl,FXint pr,FXint pt,FXint pb,
            FXint hs,FXint vs) :
 FXTopWindow(ap,name,ic,mi, opts, x,y,w,h, pl,pr,pt,pb, hs,vs)
{
  FXVerticalFrame *vbox;
  flags|=FLAG_ENABLED|FLAG_SHOWN; 

  vbox= new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  FXMenuBar *bar;
  bar= new FXMenuBar(vbox, LAYOUT_FILL_X);

  chapters_pane= new FXMenuPane(this);
  sections_pane= new FXMenuPane(this);

  new FXMenuTitle(bar,"Chapters", NULL, chapters_pane);
  sections_mtitle= new FXMenuTitle(bar,"Sections", NULL, sections_pane);

  new FYMenuCommand(bar, "Contents", NULL, 0, show_toc2, 0);
  new FYMenuCommand(bar, "Full", NULL, 0, show_toc0, 0);

  canvas= new FYGfxCanvas(vbox, FRAME_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_Y);
  

  const char *normal_font, *fixed_font;
  normal_font= "Dejavu Sans"; fixed_font= "Mono";

  canvas->define_font(HFO_NORMAL, CG_normal_font, CG_normal_size, "");
  canvas->define_font(HFO_BOLD, CG_normal_font, CG_normal_size, "B");
  canvas->define_font(HFO_ITALIC, CG_normal_font, CG_normal_size, "I");
  canvas->define_font(HFO_BIG, CG_normal_font, CG_normal_big, "");

  canvas->define_font(HFO_FNORMAL, CG_fixed_font, CG_fixed_size, "");
  canvas->define_font(HFO_FBOLD, CG_fixed_font, CG_fixed_size, "B");
  canvas->define_font(HFO_FITALIC, CG_fixed_font, CG_fixed_size, "I");
  canvas->define_font(HFO_FBIG, CG_fixed_font, CG_fixed_big, "");

  canvas->define_font(HFO_SECHEA4, CG_normal_font, CG_sechea4, "B");
  canvas->define_font(HFO_SECHEA3, CG_normal_font, CG_sechea3, "B");
  canvas->define_font(HFO_SECHEA2, CG_normal_font, CG_sechea2, "B");
  canvas->define_font(HFO_SECHEA1, CG_normal_font, CG_sechea1, "B");

  canvas->define_color(HCO_BG, 0xffffff);
  canvas->define_color(HCO_FG, 0);
 
  canvas->define_color(HCO_LINK, CG_link);
  canvas->define_color(HCO_REDBG, CG_red);
  canvas->define_color(HCO_HIBG, CG_hilight);
  canvas->define_color(HCO_CODEBG, CG_code);
  canvas->define_color(HCO_SHELLBG, CG_shell);
  canvas->define_color(HCO_OUTPUTBG, CG_output);
//uu-cons
}


FYHelpWin::~FYHelpWin()
{
// uu-des
}

// uu-chapsec
void FYHelpWin::menu_add_chapter(const char *title, int linkno)
{
  FYMenuCommand *cmd;
  cmd= new FYMenuCommand(chapters_pane, title, NULL, 0, follow_link, linkno);
  cmd->create();
}


void FYHelpWin::set_section_menu(const sipair_t *me)
{
  int i;
  FYMenuCommand *cmd;
  if (sections_def==me) return ;
  FXWindow *ww;
  while((ww=sections_pane->getFirst()))
    delete ww;
  sections_def= me;
  for(i=0;me[i].Vs;i++)
  {
     cmd= new FYMenuCommand(sections_pane, me[i].Vs, 
                            NULL, 0, follow_link, me[i].Vi);
     cmd->create();
  }
  if (i==0) 
    sections_mtitle->disable();
  else
    sections_mtitle->enable();
  sections_pane->recalc();
}

